function [wopt,dopt] = mssnr(h,Nb,tLw,dmin,dmax)
% [wopt,dopt] = mssnr(h,Nb,tLw,dmin,dmax)
%    wopt = MSSNR TEQ
%    dopt = optimal delay
%    h = channel
%    Nb = window length = CP length + 1
%    tLw = TEQ length
%    [dmin:dmax] = range of delays to try
%       delays start at zero, not 1
%
% Written by Rick Martin
% Cornell University Blind Equalization Research Group

% intialize variables
h = h(:);
lambdaopt = 0;
H = convmtx(h,tLw);
C = H'*H;

for delay = dmin:dmax % for each delay to be searched
   Hwin = H(delay+1:delay+Nb,:);
   B = Hwin'*Hwin;
   [evecs,evals] = eig(B,C);
   [lambda,ind] = max(diag(evals));
   if lambda > lambdaopt
      % save the TEQ, delay, and eigenvalue
      wopt = evecs(:,ind);
      dopt = delay;
      lambdaopt = lambda;
   end
end
