function [J,ind]=Jssnr(c,nu,flag,dmin,dmax)
% [J,ind] = Jssnr(c,nu,flag,dmin,dmax)
%
%     c = FIR filter
%     nu = window length - 1
%     ind = index of first tap in window (start at 1, not 0)
%     [dmin,dmax] = range of values of "ind" to try
%     dmin should be at least 1
%     dmax should be at most length(c)-nu
%     flag: if set to 0, then
%               (energy in window)
%     J = max -----------------------
%             (energy outside window)
%
%           if set to 1, then
%     J = min (energy outside window)
%      
%     Written by Rick Martin on 02/07/2002
%     Last updated on 08/08/2002
%     Cornell University Blind Equalization Research Group
%
%     Copyright 2002 Cornell University, All Rights Reserved
%
%     SOFTWARE shall mean "Jssnr.m", Cornell Research Foundation 
%     ("CRF") Docket D-3006.  SOFTWARE is made available to allow
%     non-commercial and research use of SOFTWARE.  Cornell
%     University specifically reserves all commercial rights to
%     SOFTWARE and these rights may be licensed by CRF on behalf   
%     of Cornell University to third parties.
%
%     CRF provides SOFTWARE on an ``as is'' basis.  CRF does not
%     warrant, guarantee, or make any representations regarding 
%     the use or results of SOFTWARE with respect to its
%     correctness, accuracy, reliability or performance.  The
%     entire risk of the use and performance of SOFTWARE is  
%     assumed by EVALUATOR.  ALL WARRANTIES INCLUDING, WITHOUT
%     LIMITATION, ANY WARRANTY OF FITNESS  FOR A PARTICULAR   
%     PURPOSE OR MERCHANTABILITY ARE HEREBY EXCLUDED.

L = length(c);

% determine allowed window choices
if nargin == 3
   dmin = 1; dmax = L-nu;
end
dmin = min(max(dmin,1),L-nu);     % in [1, L-nu]
dmax = min(max(dmax,dmin),L-nu);  % in [dmin, L-nu]

% evaluate cost function for all allowed window choices
Js = zeros(1,dmax-dmin+1);
E = sum(abs(c).^2);
E_in = sum(abs(c(dmin:dmin+nu)).^2);
E_out = E - E_in;
if flag
   Js(1) = E_out;
else
   Js(1) = E_in/E_out;
end
for win=dmin+1:dmax
   E_in  = E_in - abs(c(win-1))^2 + abs(c(win+nu))^2;
   E_out = E - E_in;
   if flag
      Js(win-dmin+1) = E_out;
   else
      Js(win-dmin+1) = E_in/E_out;
   end
end;

% determine index of window with best cost
if flag
   [J,ind] = min(Js);
else
   [J,ind] = max(Js);
end
ind = ind+dmin-1;
