function bmap = bitrate(c,w,nu,dopt,ssx,ssn)
% bmap = bitrate(c,w,nu,dopt,ssx,ssn)
%     c = effective channel
%     w = TEQ
%     nu = cyclic prefix length
%     dopt = starting index of window - 1
%     ssx = signal power
%     ssn = noise power
%
%     Calculates the achievable bit rate for
%     a given baseband transmission environment
%
%     Modified from code in the DMT-TEQ toolbox,
%     provided by the University of Texas at Austin, ESPL
%
%     Modifications made by Rick Martin, February 2002
%     Last updated on 05/22/2003
%     Cornell University Blind Equalization Research Group

N = 512;           % FFT size
margin = 6;        % in dB
codingGain = 4.2;  % in dB
snrgap = 9.8;      % in dB
minbits = 0;       % minimum bits per tone
maxbits = 100;     % maximum bits per tone

inputSpec = ssx * ones(1,N/2+1);
noiseSpec = ssn * ones(1,N/2+1);

hw = c;
win = zeros(length(hw),1);
win(dopt+1:dopt+nu+1) = ones(1,nu+1);

% signal path impulse response
hwin = hw.*win;
%hwin = hwin - mean(hwin); % what the hell is this?

% ISI path impulse response
hwout = hw.*(1-win);
% hwout = hwout - mean(hwout); % ditto

% TEQ frequency response
Fwu = fft(w,N)*sqrt(N);         Fw = Fwu(1:N/2+1).';
Fhwinu = fft(hwin,N)*sqrt(N);   Fhwin = Fhwinu(1:N/2+1).';
Fhwoutu = fft(hwout,N)*sqrt(N); Fhwout = Fhwoutu(1:N/2+1).';

% noise power spectrum
colorNoiseaft = noiseSpec.*abs(Fw).^2;
% ISI power spectrum
isiaft = inputSpec.*abs(Fhwout).^2;
% signal power spectrum
signalaft = inputSpec.*abs(Fhwin).^2;
% subband SNRs after equalization
SNRi = signalaft./( colorNoiseaft + isiaft );

% compute the capacity gap
gap = 10^((snrgap + margin - codingGain)/10);

% calculate bit allocation
bmap = log2(1 + SNRi/gap);

% round, then limit the constellation size
bmap = min( max(floor(bmap),minbits), maxbits);
