% fig5.m
% (test_symmmse.m)
% Compares the symmetric MMSE design to the normal MMSE design
%
% Written by Rick Martin, 05/22/2003
% Last updated on 05/22/2003
% Cornell University Blind Equalization Research Group

nu = 32;               % CP length
Nb = nu+1;             % TIR length
Nws = [3:128];         % range of TEQ lengths to try
%Nws = [3:21];          % range of TEQ lengths to try
numNws = length(Nws);  % number of TEQ lengths to try
SNRdb = 60;            % signal to noise ratio
SNRlinear = 10^(SNRdb/10);
ssx = 1;               % transmitted signal power
ssn = ssx/SNRlinear;   % received noise power

MMSEopt = zeros(numNws,8);
MMSEsym = zeros(numNws,8);
BRopt = zeros(numNws,8);
BRsym = zeros(numNws,8);

%csanum = 1;            % CSA loop to use
hh = waitbar(0,'Iterating over CSA loops and TEQ lengths');
for csanum=1:8

eval(['load ../channels/csaloop',num2str(csanum),'.time']);
eval(['h = csaloop',num2str(csanum),'(:,2);']);
h = h/norm(h);         % normalize for nice SNR computation
Lh = length(h);        % channel length
Dmin = 0;              % minimum delay to check
Dmax_temp = Lh-nu-1;   % maximum delay to check, for Nw=1;

for iNw = 1:numNws
   Nw = Nws(iNw);
   Dmax = Dmax_temp+(Nw-1);  % longer TEQ --> larger allowable delay

   % unconstrained (optimal) MMSE TEQ design
   [wopt,bopt,dopt,MMSEopt(iNw,csanum)] = ...
      mmse_unc(h,Nb,Nw,Dmin,Dmax,ssx,ssn);
   copt = conv(h,wopt);

   % MMSE TEQ design for a symmetric TIR
   [wsym,bsym,dsym,MMSEsym(iNw,csanum)] = ...
      mmse_sym(h,Nb,Nw,Dmin,Dmax,ssx,ssn);
   csym = conv(h,wsym);

   % compute bit rates
   BRopt(iNw,csanum) = sum( bitrate(copt,wopt,nu,dopt,ssx,ssn) );
   BRsym(iNw,csanum) = sum( bitrate(csym,wsym,nu,dsym,ssx,ssn) );

   waitbar( (csanum-1)/8 + (iNw/numNws)/8, hh);
end % for iNw = 1:numNws

end % for csanum=1:8
close(hh)

Nws(18)
rel = (BRsym(18,:) ./ BRopt(18,:)) * 100;
[BRopt(18,:).'/1e6, BRsym(18,:).'/1e6, rel.']

Tsym = 246.4e-6;
BRopt = BRopt/Tsym;
BRsym = BRsym/Tsym;
BRopt_avg = mean(BRopt,2);
BRsym_avg = mean(BRsym,2);

figure(1);
plot([0:length(copt)-1],copt,'b-', [dopt:dopt+nu],bopt,'rx');
title('Traditional MMSE design');
v = axis;
axis([0 100 v(3) v(4)]);

figure(2);
plot([0:length(csym)-1],csym,'b-', [dsym:dsym+nu],bsym,'rx');
title('MMSE design with Symmetric TIR');
v = axis;
axis([0 100 v(3) v(4)]);

figure(3);
semilogy(Nws,MMSEopt,'b-', Nws,MMSEsym,'r--');
title('MMSE');

figure(4);
clf;
set(0,'DefaultAxesFontSize',14);   
subplot(2,1,1);
plot(Nws,BRopt(:,1),'b-', Nws,BRsym(:,1),'r--');
ylabel('bit rate (loop 1)','FontSize',14);
grid on;
subplot(2,1,2);
plot(Nws,BRopt(:,3),'b-', Nws,BRsym(:,3),'r--');
legend('unconstrained','symmetric TIR',4);
xlabel('TEQ length','FontSize',14);
ylabel('bit rate (loop 3)','FontSize',14);
grid on;

figure(5);
clf;
set(0,'DefaultAxesFontSize',14);
plot(Nws,BRopt_avg/1e6,'b-', Nws,BRsym_avg/1e6,'r--');
v = axis;
axis([1 Nws(end) v(3) v(4)]);
legend('unconstrained','symmetric TIR',4);
xlabel('TEQ length','FontSize',14);
ylabel('bit rate (average), Mbps','FontSize',14);
grid on;
